@@ -30,7 +30,7 @@ var (
 
 // Create a file and open it with the flags passed in
 func rwHandleCreateFlags(t *testing.T, create bool, filename string, flags int) (r *fstest.Run, vfs *VFS, fh *RWFileHandle) {
-	opt := vfscommon.DefaultOpt
+	opt := vfscommon.Opt
 	opt.CacheMode = vfscommon.CacheModeFull
 	opt.WriteBack = writeBackDelay
 	r, vfs = newTestVFSOpt(t, &opt)
@@ -346,6 +346,9 @@ func TestRWFileHandleWriteAt(t *testing.T) {
 		return n
 	}
 
+	// Name
+	assert.Equal(t, "file1", fh.Name())
+
 	// Preconditions
 	assert.Equal(t, int64(0), offset())
 	assert.True(t, fh.opened)
@@ -631,7 +634,7 @@ func testRWFileHandleOpenTest(t *testing.T, vfs *VFS, test *openTest) {
 func TestRWFileHandleOpenTests(t *testing.T) {
 	for _, cacheMode := range []vfscommon.CacheMode{vfscommon.CacheModeWrites, vfscommon.CacheModeFull} {
 		t.Run(cacheMode.String(), func(t *testing.T) {
-			opt := vfscommon.DefaultOpt
+			opt := vfscommon.Opt
 			opt.CacheMode = cacheMode
 			opt.WriteBack = writeBackDelay
 			_, vfs := newTestVFSOpt(t, &opt)
@@ -682,7 +685,7 @@ func TestRWFileModTimeWithOpenWriters(t *testing.T) {
 }
 
 func TestRWCacheRename(t *testing.T) {
-	opt := vfscommon.DefaultOpt
+	opt := vfscommon.Opt
 	opt.CacheMode = vfscommon.CacheModeFull
 	opt.WriteBack = writeBackDelay
 	r, vfs := newTestVFSOpt(t, &opt)
@@ -711,3 +714,47 @@ func TestRWCacheRename(t *testing.T) {
 	assert.False(t, vfs.cache.Exists("rename_me"))
 	assert.True(t, vfs.cache.Exists("i_was_renamed"))
 }
+
+// Test the cache reading a file that is updated externally
+//
+// See: https://github.com/rclone/rclone/issues/6053
+func TestRWCacheUpdate(t *testing.T) {
+	opt := vfscommon.Opt
+	opt.CacheMode = vfscommon.CacheModeFull
+	opt.WriteBack = writeBackDelay
+	opt.DirCacheTime = fs.Duration(100 * time.Millisecond)
+	r, vfs := newTestVFSOpt(t, &opt)
+
+	if r.Fremote.Precision() == fs.ModTimeNotSupported {
+		t.Skip("skip as modtime not supported")
+	}
+
+	const filename = "TestRWCacheUpdate"
+
+	modTime := time.Now().Add(-time.Hour)
+	for i := 0; i < 10; i++ {
+		modTime = modTime.Add(time.Minute)
+		// Refresh test file
+		contents := fmt.Sprintf("TestRWCacheUpdate%03d", i)
+		// Increase the size for second half of test
+		for j := 5; j < i; j++ {
+			contents += "*"
+		}
+		file1 := r.WriteObject(context.Background(), filename, contents, modTime)
+		r.CheckRemoteItems(t, file1)
+
+		// Wait for directory cache to expire
+		time.Sleep(time.Duration(2 * opt.DirCacheTime))
+
+		// Check the file is OK via the VFS
+		data, err := vfs.ReadFile(filename)
+		require.NoError(t, err)
+		require.Equal(t, contents, string(data))
+
+		// Check Stat
+		fi, err := vfs.Stat(filename)
+		require.NoError(t, err)
+		assert.Equal(t, int64(len(contents)), fi.Size())
+		fstest.AssertTimeEqualWithPrecision(t, filename, modTime, fi.ModTime(), r.Fremote.Precision())
+	}
+}
